"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var sp_core_library_1 = require("@microsoft/sp-core-library");
var FetchProvider_1 = require("../httpClient/FetchProvider");
var AadHttpClientResponse_1 = tslib_1.__importDefault(require("./AadHttpClientResponse"));
var AadTokenProviders_1 = tslib_1.__importDefault(require("../oauthTokenProvider/AadTokenProviders"));
var HttpClientHelper_1 = tslib_1.__importDefault(require("../httpClient/HttpClientHelper"));
var AadHttpClientConfiguration_1 = require("./AadHttpClientConfiguration");
var ICacheDataProvider_1 = require("../caching/ICacheDataProvider");
var CacheDataProviderService_1 = require("../caching/CacheDataProviderService");
var CacheManager_1 = require("../caching/CacheManager");
/**
 * AadHttpClient is used to perform REST calls against an Azure AD Application.
 *
 * @remarks
 * For communicating with SharePoint, use the {@link SPHttpClient} class instead.
 * For communicating with Microsoft Graph, use the {@link @microsoft/sp-http-msgraph#MSGraphClient} class.
 *
 * @public
 * @sealed
 */
var AadHttpClient = /** @class */ (function () {
    /**
     *
     * @param serviceScope - The service scope is needed to retrieve some of the class's internal components.
     * @param resourceEndpoint - The resource for which the token should be obtained.
     * @param options - Configuration options for the request to get an access token.
     *
     */
    function AadHttpClient(serviceScope, resourceEndpoint, options) {
        var _this = this;
        sp_core_library_1.Validate.isNotNullOrUndefined(serviceScope, 'serviceScope');
        sp_core_library_1.Validate.isNotNullOrUndefined(resourceEndpoint, 'resourceUrl');
        this._resourceUrl = resourceEndpoint;
        this._serviceScope = serviceScope;
        this._aadTokenProvider = (options && options.tokenProvider) || AadTokenProviders_1.default.configurable;
        this._aadTokenConfiguration = options && options.configuration;
        this._useCachedToken = options && options.useCachedToken !== undefined ? options.useCachedToken : true;
        serviceScope.whenFinished(function () {
            _this._fetchProvider = serviceScope.consume(FetchProvider_1.fetchProviderServiceKey);
        });
    }
    AadHttpClient.prototype.fetch = function (url, configuration, options, cacheConfiguration) {
        var _this = this;
        // Default fetch behavior - No caching
        // If no caching options are provided, or If Cache Provider is not present continue to API call
        if (!cacheConfiguration || !this._cacheDataProvider) {
            return this._fetch(url, configuration, options, this._useCachedToken).then(function (originalResponse) {
                if (originalResponse.status === 403) {
                    return _this._fetch(url, configuration, options, false);
                }
                else {
                    return originalResponse;
                }
            });
        }
        var cacheKey = this._cacheDataProvider.getCacheKey(ICacheDataProvider_1.CacheKeyType.HttpGenerated, url, configuration, options);
        // Fetch data and cache according to the cacheConfiguration
        return CacheManager_1.CacheManager.fetchAndCache(cacheKey, this._cacheDataProvider, cacheConfiguration, function () {
            return _this._fetch(url, configuration, options, _this._useCachedToken);
        });
    };
    AadHttpClient.prototype.get = function (url, configuration, options, cacheConfiguration) {
        var forcedGetOptions = HttpClientHelper_1.default.overrideHttpMethod(options, 'GET');
        if (cacheConfiguration) {
            return this.fetch(url, configuration, forcedGetOptions, cacheConfiguration);
        }
        else {
            return this.fetch(url, configuration, forcedGetOptions);
        }
    };
    /**
     * Calls fetch(), but sets the method to "POST".
     *
     * @param url - The endpoint URL that fetch will be called on.
     * @param configuration - Determines the default behavior of HttpClient; normally this should
     *   be the latest version number from HttpClientConfigurations.
     * @param options - Additional options that affect the request.
     * @returns A promise that will return the result.
     */
    AadHttpClient.prototype.post = function (url, configuration, options) {
        return this.fetch(url, configuration, HttpClientHelper_1.default.overrideHttpMethod(options, 'POST'));
    };
    Object.defineProperty(AadHttpClient.prototype, "_cacheDataProvider", {
        /**
         * Gets the cache data provider
         */
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        get: function () {
            if (!this._cacheProvider) {
                this._cacheProvider = this._serviceScope.consume((0, CacheDataProviderService_1.getCacheDataProviderServiceKey)())._provider;
            }
            return this._cacheProvider;
        },
        enumerable: false,
        configurable: true
    });
    AadHttpClient.prototype._fetch = function (url, configuration, options, useCachedToken) {
        var _this = this;
        var startTimer = performance.now();
        // We can only use a custom configuration for the original AADTokenProvider right now.
        // We don't want to support this customization long term.
        var tokenFetchPromise;
        if (this._aadTokenConfiguration && this._aadTokenProvider._getTokenInternal) {
            tokenFetchPromise = this._aadTokenProvider._getTokenInternal(this._resourceUrl, this._aadTokenConfiguration, useCachedToken);
        }
        else {
            tokenFetchPromise = this._aadTokenProvider.getToken(this._resourceUrl, useCachedToken);
        }
        return tokenFetchPromise.then(function (token) {
            var endTimer = performance.now();
            // Constructing a Headers object with undefined throws an exception in IE/Edge
            options.headers = options.headers ? new Headers(options.headers) : new Headers();
            options.headers.set('Authorization', 'Bearer ' + token);
            return HttpClientHelper_1.default.fetchCore(configuration, new Request(url, options), _this._serviceScope, _this._fetchProvider, AadHttpClient._className).then(function (response) {
                return new AadHttpClientResponse_1.default(response, endTimer - startTimer);
            });
        });
    };
    /**
     * The standard predefined AadHttpClientConfiguration objects for use with
     * the AadHttpClient class.
     */
    AadHttpClient.configurations = AadHttpClientConfiguration_1.predefinedConfigurations;
    AadHttpClient._className = 'AadHttpClient';
    return AadHttpClient;
}());
exports.default = AadHttpClient;
//# sourceMappingURL=AadHttpClient.js.map